/****************************************************************************/
/*                                                                          */
/*  Dc_File.c :  Module de gestion lecture/criture de donnes.             */
/*                                                                          */
/****************************************************************************/
/*  Auteur : Franois GAUTHIER   *   Cooperteam   *   Date : Septembre 2010 */
/****************************************************************************/

#include "windows.h"

#include <stdio.h>
#include <stdlib.h>

#include "Dc_Memory.h"
#include "Dc_Shared.h"
#include "Dc_File.h"

static void GetUserPreferencesFilePath(char *);

/*************************************************************************/
/* LoadUserPreferences() : Chargement des dernires donnes utilisateur. */
/*************************************************************************/
void LoadUserPreferences(struct window_settings *current_settings)
{
  FILE *fd;
  long file_size;
  int  length, nb_read;
  char *begin;
  char *next_sep;
  char *next_field_flag;
  char *file_buffer;
  char file_path[BUFFER_SIZE];

  /* Initilisation : Valeurs par dfaut */
  current_settings->client_area = 0;
  strcpy(current_settings->top,""); 
  strcpy(current_settings->bottom,""); 
  strcpy(current_settings->left,""); 
  strcpy(current_settings->right,""); 
  current_settings->keep_row = 0;
  strcpy(current_settings->row,""); 
  current_settings->keep_column = 0;
  strcpy(current_settings->column,""); 
  current_settings->use_gif = 0;
  strcpy(current_settings->folder_path,"c:\\Temp\\Pictures\\"); 
  strcpy(current_settings->file_name,"img_");
  strcpy(current_settings->buffer_nb_last_file,"0001");
  current_settings->set_auto = 0;
  strcpy(current_settings->delay,"");

  /* Recupre le chemin du fichier */
  GetUserPreferencesFilePath(file_path);

  /* Ouverture du fichier */
  fd = fopen(file_path,"r");
  if(fd == NULL)
    return;
  
  /* Dtermine la taille du fichier */
  fseek(fd,0L,SEEK_END);
  file_size = ftell(fd);
  fseek(fd,0L,SEEK_SET);

  /* Cration d'un buffer de la taille du fichier */
  file_buffer = (char *) calloc(1,file_size+1);
  if(file_buffer == NULL)
    {
      fclose(fd);
      return;
    }

  /* On charge l'ensemble du fichier dans le buffer */
  nb_read = fread(file_buffer,1,file_size,fd);
  file_buffer[nb_read+1] = '\0';

  /* Fermeture du fichier */
  fclose(fd);

  /** Attribution des blocs  leur champs respectifs **/
  begin = file_buffer;
  while(begin)
    { 
      /* Recherche la fin de ligne */
      next_sep = strchr(begin,'\n');
      if(next_sep != NULL)
        *next_sep = '\0';

      /* Recherche le caractre sparateur label:data "=" */
      /** On supprime les espaces avant le caractre sparateur "=" **/
      next_field_flag = strchr(begin,'=');
      if(next_field_flag == NULL)
        {
          /* ligne suivante */
          if(next_sep == NULL)
            break;
          begin = next_sep + 1;
          continue;
        }
      *next_field_flag = '\0';

      /* Nettoyage des espaces parasites */
      CleanBuffer(begin);
      CleanBuffer(next_field_flag+1);

      /** Comparaison de l'identificateur de chaine avec les identifiants de champs **/
      if(!_stricmp(begin,"ClientArea"))
        current_settings->client_area = (!_stricmp(next_field_flag+1,"1")) ? 1 : 0;
      else if(!_stricmp(begin,"Top"))
        strcpy(current_settings->top,next_field_flag+1);
      else if(!_stricmp(begin,"Bottom"))
        strcpy(current_settings->bottom,next_field_flag+1);
      else if(!_stricmp(begin,"Left"))
        strcpy(current_settings->left,next_field_flag+1);
      else if(!_stricmp(begin,"Right"))
        strcpy(current_settings->right,next_field_flag+1);
      else if(!_stricmp(begin,"KeepRow"))
        current_settings->keep_row = (!_stricmp(next_field_flag+1,"1")) ? 1 : 0;
      else if(!_stricmp(begin,"Row"))
        strcpy(current_settings->row,next_field_flag+1);
      else if(!_stricmp(begin,"KeepColumn"))
        current_settings->keep_column = (!_stricmp(next_field_flag+1,"1")) ? 1 : 0;
      else if(!_stricmp(begin,"Column"))
        strcpy(current_settings->column,next_field_flag+1);
      else if(!_stricmp(begin,"UseGif"))
        current_settings->use_gif = (!_stricmp(next_field_flag+1,"1")) ? 1 : 0;
      else if(!_stricmp(begin,"FolderPath"))
        strcpy(current_settings->folder_path,next_field_flag+1);
      else if(!_stricmp(begin,"FileName"))
        strcpy(current_settings->file_name,next_field_flag+1);
      else if(!_stricmp(begin,"Counter"))
        strcpy(current_settings->buffer_nb_last_file,next_field_flag+1);
      else if(!_stricmp(begin,"AppendTimeDate"))
        current_settings->append_timedate = (!_stricmp(next_field_flag+1,"1")) ? 1 : 0;
      else if(!_stricmp(begin,"Auto"))
        current_settings->set_auto = (!_stricmp(next_field_flag+1,"1")) ? 1 : 0;
      else if(!_stricmp(begin,"Delay"))
        strcpy(current_settings->delay,next_field_flag+1);

      /* Ligne suivante */
      if(next_sep == NULL)
        break;
      begin = next_sep + 1;
    }

  /* Libration mmoire */
  free(file_buffer);
  
  /** Un chemin correct fini par '\' **/
  length = (int) strlen(current_settings->folder_path);
  if(length > 0)
    if((current_settings->folder_path[length-1]) != '\\')
      strcat(current_settings->folder_path,"\\");  
}


/***************************************************************************/
/* UpdateUserPreferences() : Sauvegarde des dernires donnes utilisateur. */
/***************************************************************************/
void UpdateUserPreferences(struct window_settings *current_settings)
{
  int length;
  FILE *fd;
  char file_path[BUFFER_SIZE];
  
  /* Recupre le chemin du fichier */
  GetUserPreferencesFilePath(file_path);

  /* Cration du fichier */
  fd = fopen(file_path,"w");
  if(fd == NULL)
    return;

  /* Un chemin correct fini par '\' */
  length = (int) strlen(current_settings->folder_path);
  if(length > 0)
    if((current_settings->folder_path[length-1]) != '\\')
      strcat(current_settings->folder_path,"\\");

  /** Ecriture des donnes dans le fichier **/
  fprintf(fd,"ClientArea=%d\n",current_settings->client_area);
  fprintf(fd,"Top=%s\n",current_settings->top);
  fprintf(fd,"Bottom=%s\n",current_settings->bottom);
  fprintf(fd,"Left=%s\n",current_settings->left);
  fprintf(fd,"Right=%s\n",current_settings->right);
  fprintf(fd,"KeepRow=%d\n",current_settings->keep_row);
  fprintf(fd,"Row=%s\n",current_settings->row);
  fprintf(fd,"KeepColumn=%d\n",current_settings->keep_column);
  fprintf(fd,"Column=%s\n",current_settings->column);

  fprintf(fd,"UseGif=%d\n",current_settings->use_gif);

  fprintf(fd,"FolderPath=%s\n",current_settings->folder_path);
  fprintf(fd,"FileName=%s\n",current_settings->file_name);
  fprintf(fd,"Counter=%s\n",current_settings->buffer_nb_last_file);
  fprintf(fd,"AppendTimeDate=%d\n",current_settings->append_timedate);

  fprintf(fd,"Auto=%d\n",current_settings->set_auto);
  fprintf(fd,"Delay=%s\n",current_settings->delay);

  /* Fermeture du fichier */
  fclose(fd);
}


/********************************************************************************************************************/
/* GetUserPreferencesFilePath() : Construit le chemin du fichier prfrence ou du fichier de sauvegarde des images. */
/********************************************************************************************************************/
static void GetUserPreferencesFilePath(char *path_rtn)
{
  int i, last_slash, path_length;
  char buffer_file[BUFFER_SIZE];

 /* On rcupre le chemin de l'excutable courant */
  GetModuleFileName(NULL,buffer_file,BUFFER_SIZE);
  
  /* Refus des chaines nulles */
  path_length = strlen(buffer_file);  
  if(path_length == 0)
    strcpy(buffer_file,"setup.ini");
  else
    {
      /* Traitement de la chaine pour rcuprer l'emplacement du rpertoire */
      for(last_slash=-1,i=path_length-1; i>=0; i--)
        if(buffer_file[i] == '\\')
          {
            last_slash = i;
            break;
          }
           
      /* Place le caractre de fin de chaine a l'emplacement du rpertoire */
      if(last_slash != -1)
        {
          buffer_file[last_slash+1] = '\0';
          strcat(buffer_file,"setup.ini");
        }
      else
        strcpy(buffer_file,"setup.ini");
    }
  strcpy(path_rtn,buffer_file);
}

/***************************************************************************/