/*****************************************************************
 *                                                               *
 *    Dc_Gif.c : Module de gestion des fichiers GIF.             *
 *                                                               *
 *****************************************************************
 *  Auteur : Olivier ZARDINI  *  CooperTeam  *  Date : Jui 2005  *
 *****************************************************************/

#include <io.h>

#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>

#include "Dc_Gif.h"

#define BITS   12
#define HSIZE  5003            /* 80% occupancy */

#define HashTabOf(i) global->htab[i]
#define CodeTabOf(i) global->codetab[i]
#define MAXCODE(n_bits) (((code_int) 1 << (global->n_bits)) - 1)

typedef unsigned char char_type;
typedef short int code_int;	/* was int */
typedef long int count_int;
typedef unsigned char pixval;

struct global_gif
{
  int n_bits;             /* number of bits/code */
  int maxbits;            /* user settable max # bits/code */
  code_int maxcode;       /* maximum code, given n_bits */
  code_int maxmaxcode;    /* should NEVER generate this code */
  count_int htab [HSIZE];
  unsigned short codetab [HSIZE];
  code_int free_ent;  /* first unused entry */
  int Width, Height;
  int curx, cury;
  long CountDown;
  unsigned long cur_accum;
  int cur_bits;

  unsigned char *buffer;
  int g_init_bits;
  FILE* g_outfile;

  int clear_flg;

  int ClearCode;
  int EOFCode;

  int a_count;
  char accum[256];
};

static unsigned long masks[] = 
{ 
  0x0000, 0x0001, 0x0003, 0x0007, 0x000F, 0x001F, 0x003F, 0x007F, 0x00FF,
  0x01FF, 0x03FF, 0x07FF, 0x0FFF, 0x1FFF, 0x3FFF, 0x7FFF, 0xFFFF 
};

static void Putword(int,FILE *);
static void compress(int,FILE *,struct global_gif *);
static void cl_hash(register count_int,struct global_gif *);
static void output(code_int,struct global_gif *);
static void cl_block(struct global_gif *);
static void flush_char(struct global_gif *);


/**************************************************************************/
/*  GIFWriteFileFrom256Color() :  Cration d'une image GIF 256 couleurs.  */
/**************************************************************************/
int GIFWriteFileFrom256Color(unsigned char *buf, char *name, int GWidth, int GHeight, int BackGround, int Red[], int Green[], int Blue[])
{
  FILE *fp;
  int B;
  int RWidth, RHeight;
  int LeftOfs, TopOfs;
  int Resolution;
  int ColorMapSize;
  int InitCodeSize;
  int i;
  int BitsPerPixel = 8;
  struct global_gif global;

  /* Init des variables 'globales' */
  memset(&global,0,sizeof(struct global_gif));
  global.maxbits = BITS;
  global.maxmaxcode = (code_int)1 << BITS;

  /* Cration du fichier */
  fp = fopen(name,"wb");
  if(fp == NULL)
    {
      return(1);
    }

  /* Valeurs de l'entte */
  ColorMapSize = 1 << BitsPerPixel;
  global.buffer = buf;
  RWidth = global.Width = GWidth;
  RHeight = global.Height = GHeight;
  LeftOfs = TopOfs = 0;
  global.cur_accum = 0;
  global.cur_bits = 0;
  Resolution = BitsPerPixel;
  global.CountDown = (long)global.Width * (long)global.Height;
  InitCodeSize = (BitsPerPixel <= 1) ? 2 : BitsPerPixel;
  global.curx = global.cury = 0;

  /* Ecriture de l'entte */
  fwrite("GIF87a",1,6,fp);
  Putword(RWidth,fp);
  Putword(RHeight,fp);

  B=0x80;
  B |=(Resolution -1) << 5;
  B |=(BitsPerPixel - 1);
  fputc(B,fp);

  fputc(BackGround,fp);
  fputc(0,fp);
  for(i=0; i<ColorMapSize; ++i)
    {
      fputc(Red[i],fp);
      fputc(Green[i],fp);
      fputc(Blue[i],fp);
    }

  fputc(',',fp);
  Putword(LeftOfs,fp);
  Putword(TopOfs,fp);
  Putword(global.Width,fp);
  Putword(global.Height,fp);
  fputc(0x00,fp);

  /* Taille des donnes initale */
  fputc(InitCodeSize,fp);

  /* Compression des donnes */
  compress(InitCodeSize+1,fp,&global);

  /* Un zro pour finir la srie */
  fputc(0,fp);

  /* On place le terminateur */
  fputc(';',fp);

  /* Fermeture du fichier */
  fclose(fp);

  return(0);
}


static void Putword(int w, FILE *fp)
{
  fputc(w & 0xff,fp);
  fputc((w / 256) & 0xff,fp);
}


static void compress(int init_bits, FILE *outfile, struct global_gif *global)
{
  register long fcode;
  register code_int i /* = 0 */;
  register int c;
  register code_int ent;
  register code_int disp;
  register int hshift;

  /*
   * Set up the globals:  g_init_bits - initial number of bits
   *                      g_outfile   - pointer to output file
   */
  global->g_init_bits = init_bits;
  global->g_outfile = outfile;

  /* Set up the necessary values */
  global->clear_flg = 0;
  global->maxcode = MAXCODE(n_bits = global->g_init_bits);

  global->ClearCode = (1 << (init_bits - 1));
  global->EOFCode = global->ClearCode + 1;
  global->free_ent = global->ClearCode + 2;

  /* Char Init */
  global->a_count=0;

  /** Next Pixel **/
  if(global->CountDown == 0)
    ent = EOF;
  else
    {
      global->CountDown--;
      ent = *(global->buffer + global->curx + global->cury*global->Width);

      /* Bump the current X position */
      global->curx++;
      if(global->curx == global->Width)
        {
          global->curx = 0;
          global->cury++;
        }
    }

  hshift = 0;
  for(fcode = (long) HSIZE; fcode < 65536L; fcode *= 2L)
    ++hshift;
  hshift = 8 - hshift;                /* set hash code range bound */

  cl_hash((count_int) HSIZE,global);            /* clear hash table */

  output((code_int)global->ClearCode,global);

  while(1)
    {
      /** Next Pixel **/
      if(global->CountDown == 0)
        break;
      else
        {
          global->CountDown--;
          c = *(global->buffer + global->curx + global->cury*global->Width);

          /* Bump the current X position */
          global->curx++;
          if(global->curx == global->Width)
            {
              global->curx = 0;
              global->cury++;
            }
        }

      fcode = (long) (((long) c << global->maxbits) + ent);
      i = (((code_int)c << hshift) ^ ent);    /* xor hashing */

      if(HashTabOf(i) == fcode)
        {
          ent = CodeTabOf(i);
          continue;
        }
      else if((long)HashTabOf (i) < 0)      /* empty slot */
        goto nomatch;
      disp = HSIZE - i;           /* secondary hash (after G. Knott) */
      if(i == 0)
        disp = 1;
probe:
      if((i -= disp) < 0)
        i += HSIZE;

      if(HashTabOf(i) == fcode)
        {
          ent = CodeTabOf(i);
          continue;
        }
      if((long)HashTabOf(i) > 0)
        goto probe;
nomatch:
      /**************/
      /**  Output  **/
      global->cur_accum &= masks[global->cur_bits];

      if(global->cur_bits > 0)
        global->cur_accum |= ((long)((code_int)ent) << global->cur_bits);
      else
        global->cur_accum = (code_int) ent;
      global->cur_bits += global->n_bits;

      while(global->cur_bits >= 8)
        {
          /* Char Out */
          global->accum[global->a_count++] = (unsigned int)(global->cur_accum & 0xff);
          if(global->a_count >=254)
            flush_char(global);

          global->cur_accum >>= 8;
          global->cur_bits -= 8;
        }

      /* If the next entry is going to be too big for the code size, */
      /* then increase it, if possible. */
      if(global->free_ent > global->maxcode || global->clear_flg)
        {
          if(global->clear_flg)
            {
              global->maxcode = MAXCODE (n_bits = global->g_init_bits);
              global->clear_flg = 0;
            } 
          else
            {
              ++global->n_bits;
              if(global->n_bits == global->maxbits)
                global->maxcode = global->maxmaxcode;
              else
                global->maxcode = MAXCODE(n_bits);
            }
        }
	
      if(((code_int) ent) == global->EOFCode)
        {
          /* At EOF, write the rest of the buffer. */
          while(global->cur_bits > 0)
            {
              /* Char Out */
              global->accum[global->a_count++] = (unsigned int)(global->cur_accum & 0xff);
              if(global->a_count >=254)
                flush_char(global);

              global->cur_accum >>= 8;
              global->cur_bits -= 8;
            }

          flush_char(global);
          fflush(global->g_outfile);	
        }

      ent = c;
      if(global->free_ent < global->maxmaxcode)
        {
          CodeTabOf(i) = global->free_ent++; /* code -> hashtable */
          HashTabOf(i) = fcode;
        } 
      else
        cl_block(global);
    }

  /* Put out the final code. */
  output((code_int)ent,global);
  output((code_int)global->EOFCode,global);
}


static void output(code_int code, struct global_gif *global)
{
  global->cur_accum &= masks[ global->cur_bits ];

  if(global->cur_bits > 0)
    global->cur_accum |= ((long)code << global->cur_bits);
  else
    global->cur_accum = code;

  global->cur_bits += global->n_bits;

  while(global->cur_bits >= 8)
    {
      /* Char Out */
      global->accum[global->a_count++] = (unsigned int)(global->cur_accum & 0xff);
      if(global->a_count >=254)
        flush_char(global);

      global->cur_accum >>= 8;
      global->cur_bits -= 8;
    }

  /*
   * If the next entry is going to be too big for the code size,
   * then increase it, if possible.
   */
  if(global->free_ent > global->maxcode || global->clear_flg)
    {
      if(global->clear_flg)
        {
          global->maxcode = MAXCODE (n_bits = global->g_init_bits);
          global->clear_flg = 0;
        } 
      else
        {
          ++global->n_bits;
          if(global->n_bits == global->maxbits)
            global->maxcode = global->maxmaxcode;
          else
            global->maxcode = MAXCODE(n_bits);
        }
    }
	
  if(code == global->EOFCode)
    {
      /* At EOF, write the rest of the buffer. */
      while(global->cur_bits > 0)
        {
          /* Char Out */
          global->accum[global->a_count++] = (unsigned int)(global->cur_accum & 0xff);
          if(global->a_count >=254)
            flush_char(global);

          global->cur_accum >>= 8;
          global->cur_bits -= 8;
        }

      flush_char(global);
      fflush(global->g_outfile);	
    }
}


static void cl_block(struct global_gif *global)
{
  cl_hash((count_int)HSIZE,global);
  global->free_ent=global->ClearCode+2;
  global->clear_flg=1;

  output((code_int)global->ClearCode,global);
}


static void cl_hash(register count_int hsize, struct global_gif *global)
{
  register count_int *htab_p = global->htab+hsize;

  register long i;
  register long m1 = -1L;

  i = hsize - 16;

  do{
      *(htab_p-16) = m1;
      *(htab_p-15) = m1;
      *(htab_p-14) = m1;
      *(htab_p-13) = m1;
      *(htab_p-12) = m1;
      *(htab_p-11) = m1;
      *(htab_p-10) = m1;
      *(htab_p-9) = m1;
      *(htab_p-8) = m1;
      *(htab_p-7) = m1;
      *(htab_p-6) = m1;
      *(htab_p-5) = m1;
      *(htab_p-4) = m1;
      *(htab_p-3) = m1;
      *(htab_p-2) = m1;
      *(htab_p-1) = m1;

      htab_p-=16;
	} while((i-=16) >=0);

  for(i+=16; i>0; i--)
    *--htab_p=m1;
}


static void flush_char(struct global_gif *global)
{
  if(global->a_count > 0)
    {
      fputc(global->a_count,global->g_outfile);
      fwrite(global->accum,1,global->a_count,global->g_outfile);
      global->a_count=0;
    }
}

/**************************************************************************/
