/*****************************************************************************/
/*                                                                           */
/*  Dc_Memory.c :  Module de gestion des ressources mmoire.                 */
/*                                                                           */
/*****************************************************************************/
/*  Auteur : Franois GAUTHIER   *   Cooperteam   *   Date : Septembre 2010  */
/*****************************************************************************/

#include <windows.h>

#include "Dc_Memory.h"
#include "Dc_Shared.h"

static void mem_free_windowtitle(struct window_title *);
static void mem_free_windowsettings(struct window_settings *);
static void mem_free_windowmain(struct window_main *);
static int GetSmallerElement(struct window_title **,int);

/**************************************************************/
/*  my_Memory() :  Fonction de gestion des resources mmoire. */
/**************************************************************/
void my_Memory(int code, void *param_1, void *param_2)
{
  int i, nb, smaller_indice;
  static int nb_window;
  static struct window_title *first_window;
  static struct window_title *last_window;
  static struct window_title *active_window;
  static struct window_settings *current_settings;
  static struct window_margins *current_margins;
  static struct window_main *current_main;
  struct window_title **tab_title;
  struct window_title *current_window;
  struct window_title *next_window;
    
  switch(code)
    {
      case MEMORY_INIT :
        nb_window = 0;
        first_window = NULL;
        last_window = NULL; 
        current_settings = NULL;
        current_main = NULL;
        break;
        
      case MEMORY_FREE :
        my_Memory(MEMORY_FREE_WINDOWTITLE,NULL,NULL);
        mem_free_windowsettings(current_settings);
        mem_free_windowmain(current_main);
        break;  
        
       /**************************************/
       /* Gestion de la stucture window_main */
       /**************************************/
      case MEMORY_SET_WINDOWMAIN :
         current_main = (struct window_main *) param_1;
         break;

      case MEMORY_GET_WINDOWMAIN:
        *((struct window_main **) param_1) = current_main;
        break;

       /******************************************/
       /* Gestion de la stucture window_settings */
       /******************************************/
      case MEMORY_SET_WINDOWSETTINGS :
         current_settings = (struct window_settings *) param_1;
         break;

      case MEMORY_GET_WINDOWSETTINGS :
         *((struct window_settings **) param_1) = current_settings;
         break;

       /*************************************/
       /* Gestion des stuctures WindowTitle */
       /*************************************/
      case MEMORY_ADD_WINDOWTITLE :
         /* Rcupre la structure */
         current_window = (struct window_title *) param_1;
         if(current_window == NULL)
           break;
           
         /* Attachement de la struture */
         if(first_window == NULL)
           first_window = current_window;
         else
           last_window->next = current_window;
         last_window = current_window;
       
         /* On a une structure de plus */
         nb_window++;
         break;
         
      case MEMORY_GET_WINDOWTITLE_NB :
         *((int *)param_1) = nb_window;
         break;
       
      case MEMORY_GET_WINDOWTITLE :
         /* Init */
         *((struct window_title **)param_2) = NULL;
       
         /* Indice souhait */
         nb = *((int *)param_1);
         
         /* Vrifie l'indice : hors scope */
         if(nb <=0 || nb > nb_window)
           break;
           
         /* Parcours la liste chaine */
         for(i=1,current_window=first_window; current_window; i++,current_window=current_window->next)
           if(i == nb)
             {
               *((struct window_title **)param_2) = current_window;
               break;
             }
         break;

      case MEMORY_SET_ACTIVEWINDOW :
         /* Init */
         *((struct window_title **)param_2) = NULL;
       
         /* Indice souhait */
         nb = *((int *)param_1);
         
         /* Vrifie l'indice : hors scope */
         if(nb <=0 || nb > nb_window)
           break;
           
         /* Parcours la liste chaine et stocke la fentre active */
         for(i=1,current_window=first_window; current_window; i++,current_window=current_window->next)
           if(i == nb)
             {
               active_window = current_window;
               break;
             }
         break;

      case MEMORY_GET_ACTIVEWINDOW :
         /* Rcupre la fentre en slection */
         *((struct window_title **)param_1) = active_window;
         break;

      case MEMORY_CLEAR_ACTIVEWINDOW :
         /* Efface la structure pointe par *active_window */
         active_window = NULL;
         break;
            
      case MEMORY_SORT_WINDOWTITLE :
         /* Allocation de la mmoire pour tab_title */
         tab_title = (struct window_title **) calloc(nb_window,sizeof(struct window_title *));
         if(tab_title == NULL)
           return; 
         
         /* On rcupre les structures window_title */
         for(i=0, current_window=first_window; current_window; current_window=current_window->next, i++)
           tab_title[i] = current_window;
           
         /* On organise le tableau de pointeur par ordre croissant de titre */  
         for(i=0; i<nb_window; i++) 
           {
             smaller_indice = GetSmallerElement(&tab_title[i],nb_window-i);
             current_window = tab_title[i];
             tab_title[i] = tab_title[smaller_indice+i];
             tab_title[smaller_indice+i] = current_window;
           } 
           
         /* On relie les ->next a leur suivant tri */
         for(i=0; i<nb_window-1; i++)
            tab_title[i]->next = tab_title[i+1];
            
         /* Le dernier pointe sur NULL */   
         tab_title[nb_window-1]->next = NULL;
          
         /* Initialisation du premier et dernier lment */ 
         first_window = tab_title[0];
         last_window = tab_title[nb_window-1];
         
         /* Libration de la mmoire */
         if(tab_title != NULL)
           free(tab_title);
         break;
              
      case MEMORY_FREE_WINDOWTITLE :
       /* Libration mmoire */
       for(current_window=first_window; current_window;)
         {
           next_window = current_window->next;
           mem_free_windowtitle(current_window);
           current_window = next_window;
         }
       /* Init */
       nb_window = 0;
       first_window = NULL;
       last_window = NULL;
       break;
       
      default :
        break;
    }
}


/******************************************************************************/
/*  mem_alloc_windowmain() :  Allocation mmoire de la structure window_main. */
/******************************************************************************/
struct window_main *mem_alloc_windowmain(void)
{
  struct window_main *current_main; 

   /* Allocation Structure */
  current_main = (struct window_main *) calloc(1,sizeof(struct window_main));
  if(current_main == NULL)
    return(NULL);

  return(current_main);
}


/**************************************************************************************/
/*  mem_alloc_windowsettings() :  Allocation mmoire de la structure window_settings. */
/**************************************************************************************/
struct window_settings *mem_alloc_windowsettings(void)
{
  struct window_settings *current_settings;

  /* Allocation Structure */
  current_settings = (struct window_settings *) calloc(1,sizeof(struct window_settings));
  if(current_settings == NULL)
    return(NULL);

   /* Allocation mmoire */
  current_settings->file_name = (char *) calloc(1,BUFFER_SIZE);
  current_settings->folder_path = (char *) calloc(1,BUFFER_SIZE);
  current_settings->buffer_nb_last_file = (char *) calloc(1,BUFFER_SIZE);  
  current_settings->buffer = (char *) calloc(1,BUFFER_SIZE);
  current_settings->buffer_log = (char *) calloc(1,BUFFER_SIZE);
  
  /* Vrifications */
  if(current_settings->file_name == NULL || current_settings->folder_path == NULL ||
     current_settings->buffer_nb_last_file == NULL || current_settings->buffer == NULL ||
     current_settings->buffer_log == NULL)
    {
      mem_free_windowsettings(current_settings);
      return(NULL);
    }

  return(current_settings);
}


/******************************************************************************/
/*  mem_free_windowsettings() :  Libration mmoire de la structure settings. */
/******************************************************************************/
static void mem_free_windowsettings(struct window_settings *current_settings)
{
  if(current_settings)
    {
      if(current_settings->buffer)
        free(current_settings->buffer);

      if(current_settings->buffer_log)
        free(current_settings->buffer_log);
    
      if(current_settings->file_name)
        free(current_settings->file_name);

      if(current_settings->folder_path)
        free(current_settings->folder_path);

      if(current_settings->buffer_nb_last_file)
        free(current_settings->buffer_nb_last_file);
    
      free(current_settings);
    }
}


/*******************************************************************************/
/*  mem_free_windowtitle() :  Libration mmoire de la structure window_title. */
/*******************************************************************************/
static void mem_free_windowtitle(struct window_title *current_window)
{
  if(current_window)
    {
      if(current_window->title)
        free(current_window->title);
        
      if(current_window->exe_path)
        free(current_window->exe_path);
        
      free(current_window);
    }
}


/*****************************************************************************/
/*  mem_free_windowmain() :  Libration mmoire de la structure window_main. */
/*****************************************************************************/
static void mem_free_windowmain(struct window_main *current_main)
{
  if(current_main)
    free(current_main);
}


/****************************************************************/
/*  GetSmallerElement() : Recherche de l'lment le plus petit. */
/****************************************************************/ 
 static int GetSmallerElement(struct window_title **tab_title, int nb_windows)
 {
   int i, minimum, result_cmp;
   
   /* On sort si il n'y a pas de valeurs a trier */ 
   if(nb_windows == 0)
     return(-1);
    
   /* Boucle de tri */ 
   minimum = 0;
   for(i=0; i<nb_windows; i++)
     {
       result_cmp = _stricmp(tab_title[i]->title,tab_title[minimum]->title);
       if(result_cmp <= 0)
         minimum = i;
     }

   return(minimum);
 }
 
/***************************************************************************/
