/****************************************************************************/
/*                                                                          */
/*  Dc_Shared.c :  Bibliothque de fonctions gnriques.                    */
/*                                                                          */
/****************************************************************************/
/*  Auteur : Franois GAUTHIER   *   Cooperteam   *   Date : Septembre 2010 */
/****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <malloc.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/timeb.h>
#include <direct.h>
#include <errno.h>
#include <math.h>

#include "Dc_Shared.h"

struct one_file
{
  char *file_path;
  char *file_name;

  int length;

  struct one_file *next;
};

struct one_file **LoadTabFiles(char *,char *,int *);
struct one_file *LoadListFiles(char *,char *);
struct one_file *GetOneFile(char *,char *);
unsigned char *LoadFileData(char *,int *);
int compare_file(const void *,const void *);
void mem_free_onefile(struct one_file *);
void mem_free_onefile_list(struct one_file *);

/********************************************************************/
/*  CleanBuffer() :  Suppression des espaces inutiles d'une chaine. */
/********************************************************************/
void CleanBuffer(char *buffer)
{
  int i, length;
  
  /* Verification des paramtres */
  if(buffer == NULL)
    return;
   
  /* Calcul de la longueur de la chaine */
  length = strlen(buffer);
  if(length == 0)
    return;

  /** On supprime les ventuels espaces  la fin de la chaine **/  
  for(i=length-1; i>=0; i--) 
    if(buffer[i]!=' ' && buffer[i]!='\t' && buffer[i]!='\r' && buffer[i]!='\n')
      {
        buffer[i+1] = '\0';
        break;
      } 
    
  /** On supprime les espaces en dbut de chaine **/ 
  for(i=0; buffer[i]!='\0'; i++)   
    if(buffer[i]!=' ' && buffer[i]!='\t' && buffer[i]!='\r' && buffer[i]!='\n')
      break;   

  /* Repositionne la chaine au dbut */    
  if(buffer[i]!='\0')
    memmove(&buffer[0],&buffer[i],strlen(&buffer[i])+1);
}
 
 
/********************************************************************************/
/*  DecodeNumericAsciiValue() :  Suppression des espaces inutiles d'une chaine. */
/********************************************************************************/
 int DecodeNumericAsciiValue(char* buffer)
 {  
   int i,result;
    
   /* Elimination du buffer null */
   if(buffer == NULL)
    return(-1);
    
   /* Recherche de charactre invalides */
   for(i=0; buffer[i]!='\0'; i++)
     if(!(buffer[i]>='0' && buffer[i]<='9'))
       return(-1);
    
   /* On saute les zros de dbut de chaine */
   for(i=0; i!='\0'; i++)
     if(buffer[i]!='0')
       break;
       
   /* Si la chaine ne contient que des zros */
   if(buffer[i] == '\0')
     return(0);
     
   /* Conversion de la chaine en int */
   result = atoi(&buffer[i]);
   
   return(result);
 }
 

/*********************************************************************/
/*  BuildFolderPath() :  Cration d'une arborescence de rpertoires. */
/*********************************************************************/
int BuildFolderPath(char *folder_path)
{
  char test_directory[1024];
  int file_length, i, result,created;
  struct _stat file_stat;
    
  /* Init */
  i=0;

  /* Taille de la chaine */
  file_length = strlen(folder_path);

  /* Parcours de la chaine */
  for(i=0; i<file_length; i++)
    {
      /* Copie la chaine pour test */
      test_directory[i] = folder_path[i];

      /* Dcoupe pour tester les lments du chemin */
      if(folder_path[i] == '\\' || folder_path == '\0')
        {
          /* On supprime le backslash de fin  pour le test, sauf "c:\" */
          if(i > 2)
            {
              test_directory[i]='\0';
              /* Le nom existe-il? */
              result = _stat(test_directory,&file_stat);
              /* Si non, on le cr */
              if(result == -1)
                {
                  created = _mkdir(test_directory);
                  if(created)
                    return(-1);
                }
              /* Le chemin existe, est-ce un fichier ou un dossier */
              else if(result == 0)
                {
                  if(file_stat.st_mode & _S_IFDIR)
                    /* On remet le backslash de fin */
                    test_directory[i] = '\\';
                  else
                    break;
                }
            }
        }
    }
  return(0);
}


/*******************************************************************************/
/*  RemoveDuplicatedPictures() :  Supprime les images conscutives identiques. */
/*******************************************************************************/
int RemoveDuplicatedPictures(char *folder_path, char *file_name, int *nb_total_rtn)
{
  int i, nb_file, current_length, next_length, nb_deleted, result;
  struct one_file **tab_file = NULL;
  struct one_file *current_file;
  struct one_file *next_file;
  char folder_file_path[1024];
  unsigned char *current_data = NULL;
  unsigned char *next_data = NULL;

  /* Init */
  *nb_total_rtn = 0;

  /* Rien  faire */
  nb_deleted = 0;
  if(strlen(folder_path) == 0 || strlen(file_name) == 0)
    return(0);

  /* Vrifie le format du rpertoire */
  strcpy(folder_file_path,folder_path);
  if(strlen(folder_file_path) > 0)
    if(folder_path[strlen(folder_file_path)-1] != '\\' && folder_path[strlen(folder_file_path)-1] != '/')
      strcat(folder_file_path,"\\");

  /** Rcupre un tableau tri des images **/
  tab_file = LoadTabFiles(folder_file_path,file_name,&nb_file);
  if(tab_file == NULL)
    return(0);
  *nb_total_rtn = nb_file;

  /* Rien  faire ? */
  if(nb_file == 0 || nb_file == 1)
    {
      for(i=0; i<nb_file; i++)
        mem_free_onefile(tab_file[i]);
      free(tab_file);
      return(0);
    }

  /** Parcours du Tableau **/
  current_file = tab_file[0];
  for(i=1; i<nb_file; i++)
    {
      next_file = tab_file[i];

      /*** Si la taille est identique, on regarde les data ***/
      if(current_file->length == next_file->length)
        {
          /** Charge les data de current **/
          if(current_data == NULL)
            {
              current_data = LoadFileData(current_file->file_path,&current_length);
              if(current_data == NULL)
                {
                  /* Libration mmoire */
                  if(next_data != NULL)
                    free(next_data);
                  for(i=0; i<nb_file; i++)
                    mem_free_onefile(tab_file[i]);
                  free(tab_file);

                  /* Renvoi le nombre de fichiers supprims */
                  return(nb_deleted);
                }
            }

          /** Charge les data de Next **/
          next_data = LoadFileData(next_file->file_path,&next_length);
          if(next_data == NULL)
            {
              /* Libration mmoire */
              if(current_data != NULL)
                free(current_data);
              for(i=0; i<nb_file; i++)
                mem_free_onefile(tab_file[i]);
              free(tab_file);

              /* Renvoi le nombre de fichiers supprims */
              return(nb_deleted);
            }

          /** Compare les donnes **/
          result = !memcmp(current_data,next_data,current_file->length);
          if(result)
            {
              /* Supprime le fichier */
              _unlink(next_file->file_path);
              nb_deleted++;
              free(next_data);
              next_data = NULL;
              continue;
            }
          else
            {
              /* On va basculer current_file dans next_file */
              free(current_data);
              current_file = next_file;
              current_data = next_data;
              next_data = NULL;
              continue;
            }
        }
      else
        {
          /* On va basculer current_file dans next_file */
          if(current_data)
            free(current_data);
          current_data = next_data;
          next_data = NULL;
          current_file = next_file;
          continue;
        }
    }

  /* Libration mmoire */
  for(i=0; i<nb_file; i++)
    mem_free_onefile(tab_file[i]);
  free(tab_file);

  /* Renvoie le nombre de fichiers supprims */
  return(nb_deleted);
}

/*********************************************************************/
/*  LoadTabFiles() :  Charge tous les fichiers dans un Tableau tri. */
/*********************************************************************/
struct one_file **LoadTabFiles(char *folder_path, char *file_name, int *nb_file_rtn)
{
  int i, nb_file;
  struct one_file *first_file = NULL;  
  struct one_file **tab_file = NULL;
  struct one_file *current_file;

  /* Charge tous les fichier en mmoire */
  first_file = LoadListFiles(folder_path,file_name);
  if(first_file == NULL)
    return(NULL);

  /* Compte le nombre de fichiers */
  for(nb_file=0,current_file = first_file; current_file; current_file = current_file->next)
    nb_file++;

  /* Cration du tableau d'accs rapide */
  tab_file = (struct one_file **) calloc(nb_file,sizeof(struct one_file *));
  if(tab_file == NULL)
    return(NULL);
  for(i=0,current_file = first_file; current_file; current_file = current_file->next,i++)
    tab_file[i] = current_file;

  /* Tri les fichiers par ordre alphabtique */  
  qsort(tab_file,nb_file,sizeof(struct one_file *),compare_file);
  for(i=0; i<nb_file-1; i++)
    tab_file[i]->next = tab_file[i+1];
  tab_file[nb_file-1]->next = NULL;

  /* Renvoie le tableau */
  *nb_file_rtn = nb_file;
  return(tab_file);
}


/*********************************************************************/
/*  LoadListFiles() :  Charge tous les fichiers binaires en mmoire. */
/*********************************************************************/
struct one_file *LoadListFiles(char *folder_path, char *file_name)
{
  int rc;
  long hFile;
  int first_time = 1;
  struct _finddata_t c_file;
  char buffer_folder_path[1024];
  char buffer_file_path[1024];
  struct one_file *first_file = NULL;
  struct one_file *last_file = NULL;
  struct one_file *new_file;

  /* Allocation mmoire */
  sprintf(buffer_folder_path,"%s%s*.*",folder_path,file_name);

  /** On boucle sur tous les fichiers prsents **/
  while(1)
    {
      if(first_time == 1)
        {
          hFile = _findfirst(buffer_folder_path,&c_file);
          rc = (int) hFile;
        }
      else
        rc = _findnext(hFile,&c_file);

      /* On analyse le rsultat */
      if(rc == -1)
        break;    /* no more files */
      first_time++;

      /* On ne traite pas les dossiers **/
      if((c_file.attrib & _A_SUBDIR) == _A_SUBDIR)
        continue;

      /** On traite cette entre **/
      /* Chemin du fichier */
      sprintf(buffer_file_path,"%s%s",folder_path,c_file.name);

      /* Chargement des donnes */
      new_file = GetOneFile(buffer_file_path,c_file.name);
      if(new_file == NULL)
        {
          mem_free_onefile_list(first_file);
          return(NULL);
        }
      new_file->length = c_file.size;

      /* Rattache  la liste chaine */
      if(first_file == NULL)
        first_file = new_file;
      else
        last_file->next = new_file;
      last_file = new_file;
    }

  /* On ferme */
  _findclose(hFile);

  return(first_file);
}


/****************************************************/
/*  GetOneFile() :  Rcupre un fichier en mmoire. */
/****************************************************/
struct one_file *GetOneFile(char *file_path, char *file_name)
{
  struct one_file *current_file;

  /** Allocation mmoire **/
  current_file = (struct one_file *) calloc(1,sizeof(struct one_file));
  if(current_file == NULL)
    return(NULL);
  current_file->file_name = strdup(file_name);
  current_file->file_path = strdup(file_path);
  if(current_file->file_name == NULL || current_file->file_path == NULL)
    {
      mem_free_onefile(current_file);
      return(NULL);
    }

  /* Renvoi le fichier */
  return(current_file);
}


/****************************************************/
/*  LoadFileData() :  Charge un fichier en mmoire. */
/****************************************************/
unsigned char *LoadFileData(char *file_path, int *length_rtn)
{
  FILE *fd;
  int file_size, nb_read;
  unsigned char *data;

  /** Rcupration des donnes du fichier **/
  /* Ouverture du fichier */
  fd = fopen(file_path,"rb");
  if(fd == NULL)
    return(NULL);

  /* Taille des donnes */
  fseek(fd,0L,SEEK_END);
  file_size = ftell(fd);
  fseek(fd,0L,SEEK_SET);

  /* Allocation  du buffer */
  data = (unsigned char *) calloc(1,file_size+1);
  if(data == NULL)
    {
      fclose(fd);
      return(NULL);
    }

  /* Lecture des donnes */
  nb_read = fread(data,file_size,1,fd);

  /* Fermeture du fichier */
  fclose(fd);

  /* Renvoie les donnes et la taille */
  *length_rtn = nb_read;
  return(data);
}


/*****************************************************/
/*  compare_file() : On tri par ordre alphabtique.  */
/*****************************************************/
int compare_file(const void *arg1, const void *arg2)
{
  struct one_file *file1 = *((struct one_file **)arg1);
  struct one_file *file2 = *((struct one_file **)arg2);

  /* On compare les deux structures */
  return(stricmp(file1->file_name,file2->file_name));
}


/******************************************************************************/
/*  mem_free_onefile_list() :  Libration d'une liste de structures one_file. */
/******************************************************************************/
void mem_free_onefile_list(struct one_file *first_file)
{
  struct one_file *current_file;
  struct one_file *next_file;
  
  for(current_file=first_file; current_file; )
    {
      next_file = current_file->next;
      mem_free_onefile(current_file);
      current_file = next_file;
    }
}


/***************************************************************/
/*  mem_free_onefile() :  Libration de la structure one_file. */
/***************************************************************/
void mem_free_onefile(struct one_file *current_file)
{
  if(current_file)
    {
      if(current_file->file_path)
        free(current_file->file_path);

      if(current_file->file_name)
        free(current_file->file_name);
        
      free(current_file);
    }
}
/*******************************************************************************/
