/***************************************************************************/
/*                                                                         */
/*  Dc_Windows.c :  Module de gestion des ressources Windows.              */
/*                                                                         */
/***************************************************************************/
/*  Auteur : Franois GAUTHIER  *   Cooperteam   *   Date : Septembre 2010 */
/***************************************************************************/

#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include "Ui_DialogMain.h"
#include "Dc_Window.h"
#include "resource.h"
#include "Dc_Shared.h"
#include "Dc_Graphic.h"
#include "Dc_Memory.h"
#include "Dc_Gif.h"

/***************************************************************/
/*  my_Windows() :  Fonction de gestion des resources Windows. */
/***************************************************************/
void my_Windows(int code, void *param_1, void *param_2)
{
  static HWND dialog_main_handle;
  static HINSTANCE window_instance;
  static HWND dialog_settings_handle;

  switch(code)
    {
      /**************************************************/
      /* Gestion de l'Instance de la fentre principale */
      /**************************************************/
      case WINDOWS_SET_INSTANCE :
        window_instance = *((HINSTANCE*)param_1);
        break;

      case WINDOWS_GET_INSTANCE :
        *((HINSTANCE *)param_1) = window_instance;
        break;

      /**********************************************/
      /* Gestion du Handle de la fentre principale */
      /**********************************************/
      case WINDOWS_SET_HDIALOGMAIN :
        dialog_main_handle = *((HWND *)param_1);
        break;

      case WINDOWS_GET_HDIALOGMAIN :
        *((HWND *)param_1) = dialog_main_handle;
        break;

      /**************************************************/
      /* Gestion du Handle de la fentre de paramtrage */
      /**************************************************/
      case WINDOWS_SET_HDIALOGSETTINGS :
        dialog_settings_handle = *((HWND *)param_1);
        break;

      case WINDOWS_GET_HDIALOGSETTINGS :
        *((HWND *)param_1) = dialog_settings_handle;
        break;

      default :
        break;
    }
}


/************************************************************/
/*  my_Button() :  Fonction de gestion des boutons Windows. */
/************************************************************/
void my_Button(int code, HWND param1)
{
  HWND hItem;

  switch(code)
    {
      case BUTTON_GETPICTURE_ON :
        hItem = GetDlgItem(param1,ID_BTN_GETPICTURE);
        EnableWindow(hItem,TRUE);
      break;

      case BUTTON_GETPICTURE_OFF :
        hItem = GetDlgItem(param1,ID_BTN_GETPICTURE);
        EnableWindow(hItem,FALSE);
      break;

      case BUTTON_SELECTWINDOW_ON :
        hItem = GetDlgItem(param1,ID_BTN_SELECT_WINDOW);
        EnableWindow(hItem,TRUE);
      break;

      case BUTTON_SELECTWINDOW_OFF :
        hItem = GetDlgItem(param1,ID_BTN_SELECT_WINDOW);
        EnableWindow(hItem,FALSE);
      break;

      case CHECKBOX_TOPMOST_ON :
        hItem = GetDlgItem(param1,IDC_CHECK_SETTOPMOST);
        EnableWindow(hItem,TRUE);
      break;

      case CHECKBOX_TOPMOST_OFF :
        hItem = GetDlgItem(param1,IDC_CHECK_SETTOPMOST);
        EnableWindow(hItem,FALSE);
      break;

      case BUTTON_SETTINGS_ON :
        hItem = GetDlgItem(param1,IDC_BUTTON_SETTINGS);
        EnableWindow(hItem,TRUE);
      break;

      case BUTTON_SETTINGS_OFF :
        hItem = GetDlgItem(param1,IDC_BUTTON_SETTINGS);
        EnableWindow(hItem,FALSE);
      break;

      default :
      break;
    }
}


/****************************************************************************************************/
/*  GetWindowAsBitmap() :  Rcupre la fentre sous forme de bitmap et l'entregistre sur le disque. */
/****************************************************************************************************/
int GetWindowAsBitmap(int client_area, int top_margin, int bottom_margin, int right_margin, int left_margin, 
                      HWND hWindow, int width, int height, int keep_column, int keep_row, int *picture_width_rtn, int *picture_height_rtn,  
                      unsigned char **bitmap_data_rtn, int *nb_color_rtn, int *palette_ind)
{
  unsigned char *windows_bitmap_data;
  unsigned char *bitmap_data;
  int nb_line, i, j, x, window_height, window_width, picture_height, picture_width, nb_color, nb_byte;
  int offset_bitmap, offset_picture;
  static int session_count = 0;
  HBITMAP bitmap_handle, holdbitmap, new_bitmap;
  HWND hDlg;
  BITMAPINFO bitmap_info;
  HDC dc_screen, dc_bitmap;
  RECT capRect;
  POINT point;
  BITMAP bitmap;
  struct window_main *current_main;
  struct window_settings *current_settings;

  /* Init */
  *bitmap_data_rtn = NULL;
  window_height = 0; 
  window_width = 0;
  nb_color = 0;
  nb_byte = 4;
  my_Memory(MEMORY_GET_WINDOWMAIN,&current_main,NULL);
  my_Memory(MEMORY_GET_WINDOWSETTINGS,&current_settings,NULL);

  /* Cr le Device Context initial */
  dc_screen = CreateDC("DISPLAY",NULL,NULL,NULL);
  if(dc_screen == NULL)
    {
      /* Libration des ressources */
      DeleteDC(dc_screen);
      return (ERR_WNDASBMP_CREATEDC);
    }

  dc_bitmap = CreateCompatibleDC(dc_screen);
  if(dc_bitmap == NULL)
    {
      /* Libration des ressources */
      DeleteDC(dc_bitmap);
      DeleteDC(dc_screen);
      return (ERR_WNDASBMP_CREATECMPDC);
    }
 
  if(client_area == 1)
    {
      /* Recupre la zone client, moins les ventuelles marges */
      GetClientRect(hWindow, &capRect);
      capRect.left = capRect.left+left_margin;
      point.x = capRect.left;
      capRect.top = capRect.top+top_margin;
      point.y = capRect.top;
      ClientToScreen(hWindow,&point);
      capRect.left = point.x;
      capRect.top = point.y;
      capRect.right = capRect.right - right_margin;
      point.x = capRect.right - left_margin;
      capRect.bottom = capRect.bottom - bottom_margin;
      point.y = capRect.bottom - top_margin;
    }
  else
    {
      /* Rcupre la fentre entire */
      GetWindowRect(hWindow,&capRect);
      point.x = capRect.left;
      point.y = capRect.top;
    }
  
  /* Cration de la zone mmoire de taille correspondante (fentre entire ou zone client) */
  if(client_area == 1)
    {
      /* x et y correspondent a la zone client */
      bitmap_handle = CreateCompatibleBitmap(dc_screen,point.x,point.y);
      window_height = point.y;
      window_width = point.x;
    }
  else
    {
      /* x et y correspondent a la fentre entire */
      bitmap_handle = CreateCompatibleBitmap(dc_screen,width,height);
      window_height = height;
      window_width = width;
    }
  if(bitmap_handle == NULL)
    {
      DeleteDC(dc_bitmap);
      DeleteDC(dc_screen);
      DeleteObject(bitmap_handle);
      return (ERR_WNDASBMP_CREATECMPBMP);
    }

  /* Slectionne l'objet a traiter */
  holdbitmap = (HBITMAP) SelectObject(dc_bitmap,bitmap_handle);

  /* BitBlit transfert les informations de couleurs depuis le DC source vers le DC de destination */
  BitBlt(dc_bitmap,0,0,window_width,window_height,dc_screen,capRect.left,capRect.top,SRCCOPY);

  /* On slectionne l'objet en zone mmoire nouvellement cre */
  new_bitmap = (HBITMAP) SelectObject(dc_bitmap,holdbitmap);

  /* Rcupration des informations sur le bitmap */
  GetObject(new_bitmap,sizeof(bitmap),&bitmap);

  /* allocation mmoire (on va rcuprer en 32 bits par pixel) */
  windows_bitmap_data = (unsigned char *) calloc(1,4*window_width*window_height);
  if(windows_bitmap_data == NULL)
    {
      /* Libration des ressources */
      DeleteDC(dc_bitmap);
      DeleteDC(dc_screen);
      DeleteObject(bitmap_handle);
      DeleteObject(holdbitmap);
      DeleteObject(new_bitmap);
      return(ERR_WNDASBMP_CALLOC);
    }

  /* initialisation structure BITMAPINFO */
  memset(&bitmap_info,0,sizeof(BITMAPINFOHEADER));
  bitmap_info.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
  bitmap_info.bmiHeader.biPlanes = 1;
  bitmap_info.bmiHeader.biBitCount = 32;
  bitmap_info.bmiHeader.biWidth = window_width;
  bitmap_info.bmiHeader.biHeight = window_height*(-1);
  bitmap_info.bmiHeader.biCompression = BI_RGB;
  
  /* Rcupration des bits */
  nb_line = GetDIBits(dc_bitmap,new_bitmap,0,window_height,windows_bitmap_data,&bitmap_info,DIB_RGB_COLORS);
  if(nb_line != window_height)
    {
      /* Libration des ressources */
      DeleteDC(dc_bitmap);
      DeleteDC(dc_screen);
      DeleteObject(new_bitmap);
      return(ERR_WNDASBMP_GETDIBITS);
    }

  /*** On dtermine le nb de couleurs de l'image ***/
  nb_color = Compute8BitColors(nb_byte,window_height,window_width,windows_bitmap_data,&(palette_ind[0]));

  /** Taille de l'image finale **/
  if(keep_column == 1)
    picture_width = window_width;
  else
    {
      picture_width = window_width / keep_column;
      if(keep_column*(window_width/keep_column) < window_width)
        picture_width++;
    }
  if(keep_row == 1)
    picture_height = window_height;
  else
    {
      picture_height = window_height / keep_row;
      if(keep_row*(window_height/keep_row) < window_height)
        picture_height++;
    }

  /** Elimine le 4eme octet de chaque pixel stocks (l'image est sur 24 bits) **/
  bitmap_data = (unsigned char *) calloc(1,3*picture_width*picture_height);
  if(bitmap_data == NULL)
    return(ERR_WNDASBMP_CALLOC);

  /*** On copie 3 octets de chaque bloc de 4 (Le codage RGB est invers en BGR suivant les cas) ***/
  if(client_area == 1 && nb_color <= 32)
    {
      offset_bitmap = 0;
      offset_picture = 0;
      for(j=0; j<window_height; j+=keep_row)
        {
          for(i=0,x=0; i<(window_width*4); i+=(keep_column*4))
            {
              bitmap_data[offset_picture+x+2] = windows_bitmap_data[offset_bitmap+i];    /* B */
              bitmap_data[offset_picture+x+1] = windows_bitmap_data[offset_bitmap+i+1];  /* G */
              bitmap_data[offset_picture+x] = windows_bitmap_data[offset_bitmap+i+2];    /* R */

              /* x suivant */
              x += 3;
            }

          /* Ligne suivante */
          offset_bitmap += (keep_row*window_width)*4;
          offset_picture += picture_width*3;
        }
    }
  else
    {
      offset_bitmap = 0;
      offset_picture = 0;
      for(j=0; j<window_height; j+=keep_row)
        {
          for(i=0,x=0; i<(window_width*4); i+=(keep_column*4))
            {
              bitmap_data[offset_picture+x+2] = windows_bitmap_data[offset_bitmap+i];    /* B */
              bitmap_data[offset_picture+x+1] = windows_bitmap_data[offset_bitmap+i+1];  /* G */
              bitmap_data[offset_picture+x] = windows_bitmap_data[offset_bitmap+i+2];    /* R */

              /* x suivant */
              x += 3;
            }

          /* Ligne suivante */
          offset_bitmap += (keep_row*window_width)*4;
          offset_picture += picture_width*3;
        }
    }

  /* Libration des ressources */
  free(windows_bitmap_data);
  DeleteDC(dc_bitmap);
  DeleteDC(dc_screen);
  DeleteObject(bitmap_handle);

  /* Recupre le Handle de la fentre principale */
  my_Windows(WINDOWS_GET_HDIALOGMAIN,&hDlg,NULL);

  /* Renseigne la taille dans la structure de stockage et met a jour l'affichage */
  sprintf(current_main->picture_size,"%d x %d",picture_width,picture_height);
  SetWindowText(GetDlgItem(hDlg,IDC_STATIC_PICTURE_SIZE),current_main->picture_size);

  /* Renvoie l'image */
  *bitmap_data_rtn = bitmap_data;
  *picture_height_rtn = picture_height;
  *picture_width_rtn = picture_width;
  *nb_color_rtn = nb_color;
  return(0);
}

/***************************************************************************/