/****************************************************************************/
/*                                                                          */
/*  WindowCapture :  Capture du contenu d'une fentre dans un fichier.      */
/*                                                                          */
/****************************************************************************/
/*  Auteur : Franois GAUTHIER   *   Cooperteam   *   Date : Septembre 2010 */
/****************************************************************************/

#include <windows.h>
#include <commctrl.h>

#include <stdlib.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Window.h"
#include "resource.h"
#include "Ui_DialogSelect.h"
#include "Ui_DialogMain.h"
#include "Dc_File.h"

/*******************************************************/
/*  WinMain() :  Fonction principale de l'application. */
/*******************************************************/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE wdc_instance, LPSTR lpCmdLine, int nCmdShow)
{
  HWND hDialog;
  MSG msg;
  struct window_title *active_window;
  struct window_settings *current_settings;
  struct window_main *current_main;

  /* Init mmoire */
  my_Memory(MEMORY_INIT,NULL,NULL);

  /* Allocation des structures */
  current_settings = mem_alloc_windowsettings();
  if(current_settings == NULL)
    return(0);
  my_Memory(MEMORY_SET_WINDOWSETTINGS,current_settings,NULL);
  current_main = mem_alloc_windowmain();
  if(current_main == NULL)
    return(0);
  my_Memory(MEMORY_SET_WINDOWMAIN,current_main,NULL);

  /** Charge en mmoire les prfrences depuis le fichier / Initialise la structure **/
  LoadUserPreferences(current_settings);

  /* Initialisation de Control */
  InitCommonControls();

  /** Cration de la DialogBox Main **/
  hDialog = CreateDialog(hInstance,MAKEINTRESOURCE(IDD_DIALOG1),NULL,(DLGPROC)MainDialogProc);
  
  /* Stocke l'instance e la fentre principale */
  my_Windows(WINDOWS_SET_INSTANCE,&hInstance,NULL);
  
  /* Stocke le handle de la fenetre principale */
  my_Windows(WINDOWS_SET_HDIALOGMAIN,&hDialog,NULL);

  /* Affichage de la fentre */
  ShowWindow(hDialog,nCmdShow);

  /* Position dsire */
  SetWindowPos(hDialog,HWND_TOP,683,270,-1,-1,SWP_NOSIZE);

  /** On rentre dans la boucle des messages **/
  while(GetMessage(&msg,NULL,0,0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }

  /** Fin de l'application **/    
  /* Sauvegarde des prfrences dans le fichier Setup.ini */
  UpdateUserPreferences(current_settings);

  /** Remise de la fentre cible dans le z-order **/
  /* Rcupre le Handle de la fentre slectionne */
  my_Memory(MEMORY_GET_ACTIVEWINDOW,&active_window,NULL);

  /* Annule le Top-Most ventuel  */ 
  if(active_window != NULL)
    SetWindowPos(active_window->hWnd,HWND_NOTOPMOST,-1,-1,-1,-1,SWP_NOMOVE|SWP_NOSIZE);

  /* Fermeture de la fentre Main */
  EndDialog(hDialog,0);
  
  /* Libration des ressources mmoire */
  my_Memory(MEMORY_FREE,NULL,NULL);
  
  return((int)msg.wParam);
}

/*************************************************************************************/
