/****************************************************************************/
/*                                                                          */
/*  Ui_DialogMain.c :  Fonctions de la fentre principale.                  */
/*                                                                          */
/****************************************************************************/
/*  Auteur : Franois GAUTHIER   *   Cooperteam   *   Date : Septembre 2010 */
/****************************************************************************/

#include <windows.h>
#include <commctrl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "Dc_Shared.h"
#include "Dc_Memory.h"
#include "Dc_Window.h"
#include "Dc_File.h"
#include "resource.h"
#include "Ui_DialogSelect.h"
#include "Ui_DialogMain.h"
#include "Ui_DialogSettings.h"
#include "Dc_Graphic.h"
#include "Dc_Gif.h"

BOOL CALLBACK EnumWindowsProc(HWND,LPARAM);
LRESULT CALLBACK WndProc(HWND,UINT,WPARAM,LPARAM);

/***********************************************************************************/
/*  MainDialogProc(): Fonction de traitement des messages de la fentre DialogBox. */
/***********************************************************************************/
BOOL CALLBACK MainDialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  unsigned long start_length, start_current, start_next;
  char buffer[1024];
  char file_path[1024];
  unsigned char *bitmap_data;
  HWND hdialog_settings,hItem;
  int palette_ind[257];
  int error, status, delay, nb_color;
  int top_margin, bottom_margin, right_margin, left_margin, settings_made;
  int picture_width, picture_height, keep_row, keep_column, file_format;
  static int started, session_counter = 0, flag = 1;
  static int min = 0;
  static time_t start = 0, current = 0;
  struct _timeb picture_timebuffer;
  time_t picture_time;
  struct tm *time_date;
  char horodate[256];
  double elapsed_time;
  struct window_settings *current_settings;
  struct window_main *current_main;
  struct window_title *active_window;
 
  /* Init */
  my_Memory(MEMORY_GET_WINDOWSETTINGS,&current_settings,NULL);
  my_Memory(MEMORY_GET_WINDOWMAIN,&current_main,NULL);
  my_Memory(MEMORY_GET_ACTIVEWINDOW,&active_window,NULL);
  my_Windows(WINDOWS_GET_HDIALOGSETTINGS,&hdialog_settings,NULL);

  switch(message)
    {
      case WM_INITDIALOG :
        strcpy(buffer,"");
        sprintf(buffer,"%s%s%s",current_settings->folder_path,current_settings->file_name,current_settings->buffer_nb_last_file);
        SetWindowText(GetDlgItem(hDlg,IDC_LATEST_FILE),buffer);    
      break;
          
      case WM_COMMAND:
        if(LOWORD (wParam) == IDC_BUTTON_SETTINGS)
          { 
            /* Affichage de la fentre de paramtrage*/
            my_Windows(WINDOWS_GET_HDIALOGMAIN,&hDlg,NULL);
            settings_made = ShowPictureSettingsWindow(hDlg); 
          }
        else if(LOWORD(wParam) == ID_BTN_GETPICTURE)
          {
            /* Rcupre le Handle de la fentre slectionne */
            my_Memory(MEMORY_GET_ACTIVEWINDOW,&active_window,NULL);

            /* Si la fentre slectionne est rduite, on l'ouvre */
            status = IsIconic(active_window->hWnd);
            if(status)
              {
                OpenIcon(active_window->hWnd);
                Sleep(300); /* Laisse le temps au sytme de raffraichir l'cran: 3/10 seconde */
              }

            /** Gestion des vnements lis au Timer **/
            if(current_settings->set_auto == 1)
              {
                /* Cration et initialisation du Timer */
                delay = atoi(current_settings->delay);
                SetTimer(hDlg,ID_TIMER,delay,NULL);

                /* Le chronomtre est actif */
                hItem = GetDlgItem(hDlg,IDC_STATIC_ELAPSEDCAPTION);
                EnableWindow(hItem,TRUE);
                hItem = GetDlgItem(hDlg,IDC_STATIC_TIME);
                EnableWindow(hItem,TRUE);
                /* Le chrono dmarre */
                if(start == 0)
                  time(&start);

                /** Change le "Caption" du bouton Start/Stop **/
                if(started != 1)
                  {
                    /* Bouton : Stop */
                    SetWindowText(GetDlgItem(hDlg,ID_BTN_GETPICTURE),"Stop");
                    started = 1;
                    /* Le boutton Settings devient inactif */
                    hItem = GetDlgItem(hDlg,IDC_BUTTON_SETTINGS);
                    EnableWindow(hItem,FALSE);
                  }
                else
                  {
                    /* Bouton : Start */
                    SetWindowText(GetDlgItem(hDlg,ID_BTN_GETPICTURE),"Start");
                    started = 0;
                    /* Le timer est rinitialis */
                    SetTimer(hDlg,ID_TIMER,delay,NULL);
                    current = 0;
                    start = 0;
                    /* Rinitialisation de l'affichage chrono */
                    sprintf(current_main->elapsed_time,"%02d:%02d",00,00);
                    SetWindowText(GetDlgItem(hDlg,IDC_STATIC_TIME),current_main->elapsed_time);
                    /* Le boutton Settings devient actif */
                    hItem = GetDlgItem(hDlg,IDC_BUTTON_SETTINGS);
                    EnableWindow(hItem,TRUE);
                  } 
              }
            if(current_settings->set_auto == 1 && started == 0) /** Conditions de STOP Timer **/
              {  
                 /* Rinitialise le "Caption" et annule l'Auto-Shoot */
                 SetWindowText(GetDlgItem(hDlg,ID_BTN_GETPICTURE),"Start");
                 KillTimer(hDlg,ID_TIMER); 
                 return(0);
              }
          case WM_TIMER:   /* Pas de break */
            if(current_settings->set_auto == 1)
              {
                /* Diffrence du chrono */
                time(&current);

                /* Mise  jour du Chronomtre et criture de la valeur dans le champs */
                elapsed_time = difftime(current,start);
                if(elapsed_time > 0)
                  {
                    /* Incrmente les minutes si la seconde change */
                    if(flag)
                      {
                        min += ((int)elapsed_time%60==0) ? 1 : 0;
                        flag = 0;
                      }
                    if((int)elapsed_time%60!=0)
                      flag=1;
                    sprintf(current_main->elapsed_time,"%02d:%02d",min,(int)elapsed_time%60);
                    SetWindowText(GetDlgItem(hDlg,IDC_STATIC_TIME),current_main->elapsed_time);
                  }
              }
            
            /* Nettoyage des buffers */
            CleanBuffer(current_settings->buffer_nb_last_file);
            CleanBuffer(current_settings->file_name);
            CleanBuffer(current_settings->folder_path);
            
            /* Rcupre la valeur numrique du champ et son format */
            start_length = strlen(current_settings->buffer_nb_last_file);
            start_current = DecodeNumericAsciiValue(current_settings->buffer_nb_last_file);
            if(start_current < 0)
              {
                MessageBox(hDlg,"Invalid character","Format Error",MB_APPLMODAL);
              }
            else
              {
                /** Capture de l'image **/
                if(active_window != NULL)
                  {
                    /* On rcupre les valeurs pour couper les marges s'il y en a */
                    if(current_settings->client_area == 1)
                      {
                        top_margin = (atoi(current_settings->top));
                        bottom_margin = (atoi(current_settings->bottom));
                        right_margin = (atoi(current_settings->right));
                        left_margin = (atoi(current_settings->left));
                      }
                    else
                      {
                        top_margin = 0;
                        bottom_margin = 0;
                        right_margin = 0;
                        left_margin = 0;
                      }

                    /* On ne prend pas toutes les colonnes / toutes les lignes */
                    keep_column = (current_settings->keep_column == 1) ? atoi(current_settings->column) : 1;
                    keep_column = (keep_column == 0) ? 1 : keep_column;
                    keep_row = (current_settings->keep_row == 1) ? atoi(current_settings->row) : 1;
                    keep_row = (keep_row == 0) ? 1 : keep_row;

                    /*** Rcupre les points de l'image en True Color RGB ***/
                    _ftime(&picture_timebuffer);
                    time(&picture_time);
                    error = GetWindowAsBitmap(current_settings->client_area, top_margin, bottom_margin, right_margin, left_margin, 
                                              active_window->hWnd, active_window->delta_rc_window_width, active_window->delta_rc_window_height, keep_column, keep_row,
                                              &picture_width, &picture_height, &bitmap_data, &nb_color, &(palette_ind[0]));

                    /** Traitement des codes retour **/
                    if(error)
                      {
                        /** Code commun  toutes les erreurs **/
                        /* Le timer est rinitialis */
                        KillTimer(hDlg,ID_TIMER);
                        SetTimer(hDlg,ID_TIMER,delay,NULL);
                        current = 0;
                        start = 0;
                        /* Rinitialisation de l'affichage chrono */
                        sprintf(current_main->elapsed_time,"%02d:%02d",00,00);
                        SetWindowText(GetDlgItem(hDlg,IDC_STATIC_TIME),current_main->elapsed_time);

                        /* Le bouton redevient Start */
                        SetWindowText(GetDlgItem(hDlg,ID_BTN_GETPICTURE),"Start");
                        started = 0;
                        /* Le boutton Settings devient actif */
                        hItem = GetDlgItem(hDlg,IDC_BUTTON_SETTINGS);
                        EnableWindow(hItem,TRUE);

                        /** Message d'erreur spcifiques **/
                        if(error == ERR_WNDASBMP_CALLOC)
                          MessageBox(hDlg,"Memory Allocation Failed! Set-Auto cancelled, computer is out of memory.","ERROR MESSAGE",0);
                        else if(error == ERR_WNDASBMP_CREATEDC)
                          MessageBox(hDlg,"Windows Function CreateDC() Failed!","ERROR MESSAGE",MB_APPLMODAL);
                        else if(error == ERR_WNDASBMP_CREATECMPDC)
                          MessageBox(hDlg,"Windows Function CreateCompatibleDC() Failed!","ERROR MESSAGE",MB_APPLMODAL);
                        else if(error == ERR_WNDASBMP_CREATECMPBMP)
                          MessageBox(hDlg,"Windows Function CreateCompatibleBMP() Failed!","ERROR MESSAGE",MB_APPLMODAL);
                        else if(error == ERR_WNDASBMP_GETDIBITS)
                          MessageBox(hDlg,"Windows Function GetDIBits() Failed!","ERROR MESSAGE",MB_APPLMODAL);

                        /* On sort */
                        return(FALSE);
                      }

                    /** Dtermine le format du fichier Image (GIF ou BMP) **/
                    if(nb_color > 256 || current_settings->use_gif == 0)
                      file_format = FILE_FORMAT_BMP;
                    else
                      file_format = FILE_FORMAT_GIF;
                    
                    /** Nombre de couleurs (Color Depth) **/
                    if(nb_color <= 16)
                      SetWindowText(GetDlgItem(hDlg,IDC_STATIC_COLOR),"16");
                    else if(nb_color <= 256)
                      SetWindowText(GetDlgItem(hDlg,IDC_STATIC_COLOR),"256");
                    else
                      SetWindowText(GetDlgItem(hDlg,IDC_STATIC_COLOR),"True");
                    
                    /** Chemin du fichier **/
                    sprintf(file_path,"%s%s%s",current_settings->folder_path,current_settings->file_name,current_settings->buffer_nb_last_file);
                    if(current_settings->append_timedate == 1)
                      {
                        /* On intgre les informations d'hordatage dans le nom du fichier */
                        time_date = localtime(&picture_time);
                        sprintf(horodate,"_%04d%02d%02d_%02d%02d%02d_%02d",1900+time_date->tm_year,time_date->tm_mon+1,time_date->tm_mday,time_date->tm_hour,time_date->tm_min,time_date->tm_sec,(picture_timebuffer.millitm)/10);
                        strcat(file_path,horodate);
                      }
                    strcat(file_path,(file_format == FILE_FORMAT_BMP)?".bmp":".gif");
                    SetWindowText(GetDlgItem(hDlg,IDC_LATEST_FILE),file_path);
                    
                    /* Ecriture du field contenant les infos des GetWindowsText */
                    start_next = start_current + 1;
                    sprintf(current_settings->buffer,"%%0%dd",start_length);
                    sprintf(current_settings->buffer_nb_last_file,current_settings->buffer,start_next);
                    /* Ecriture de la valeur Counter mise a jour */
                    session_counter++;
                    sprintf(current_main->session_counter,"%d",session_counter);
                    SetWindowText(GetDlgItem(hDlg,IDC_STATIC_SESSION_COUNT),current_main->session_counter); 

                    /** Cration du chemin o enregistrer l'image **/
                    error = BuildFolderPath(file_path);
                    if(error)
                      {
                        MessageBox(hDlg,"BuildFolderPath failed, this screenshot wasn't saved", "ERROR MESSAGE",MB_APPLMODAL);
                        return(0);
                      }

                    /*** Cration de l'image (GIF ou BMP) sur disque via un Thread ddi ***/
                    CreateOnePictureOnDisk(current_settings,file_path,file_format,picture_width,picture_height,bitmap_data,nb_color,&palette_ind[0]);
                  }
              }
          }
      break;

      case WM_CLOSE :
        PostQuitMessage(0);
        break;

      default:
        return(FALSE);
    }
  return(TRUE);
}


/******************************************************************/
/*  EnumWindowsProc() :  Fonction de scan des fentres top-level. */
/******************************************************************/
BOOL CALLBACK EnumWindowsProc(HWND hWnd, LPARAM lParam) 
{
  BOOL result;
  char title[1024];
  char exe_path[1024];
  int delta_rc_window_height,delta_rc_client_height;
  int delta_rc_window_width,delta_rc_client_width;
  int length;
  struct window_title *current_window;
  WINDOWINFO window_info;
  
  /* Titre de la fentre et chemin du .exe */
  length = GetWindowText(hWnd,title,1024);
  title[length] = '\0';
  length = GetWindowModuleFileName(hWnd,exe_path,1024);
  exe_path[length] = '\0';
  
  /* On ne prend que les fentres visibles */
  result = IsWindowVisible(hWnd);
    if(!result)
      return(TRUE);
    
  /* On ne prend que les fentres utiles */
  if(strlen(title) == 0)
    return(TRUE);
    
  /* On limine les .dll */
  if(strlen(exe_path) > 4)
    if(!_stricmp(&exe_path[strlen(exe_path)-4],".dll"))
      return(TRUE);
   
  /* On ne prend que les fentre de taille minimum */
  memset(&window_info,0,sizeof(WINDOWINFO));
  result = GetWindowInfo(hWnd,&window_info);
  
  /* On fait le delta de la taille des fentre */
  delta_rc_window_height = window_info.rcWindow.bottom - window_info.rcWindow.top;
  delta_rc_window_width = window_info.rcWindow.right - window_info.rcWindow.left;
  
  /* On fait le delta de la taille de la zone client */
  delta_rc_client_height = window_info.rcClient.bottom - window_info.rcClient.top;
  delta_rc_client_width = window_info.rcClient.right - window_info.rcClient.left;
  if((delta_rc_window_height < 10  && delta_rc_window_width < 20) && (delta_rc_client_height < 5 && delta_rc_client_width < 10))
    return(TRUE);
  
  /* Allocation mmoire */
  current_window = (struct window_title *) calloc(1,sizeof(struct window_title));
  if(current_window == NULL)
    return(TRUE);

  /* Remplissage structure */
  current_window->hWnd = hWnd;
  memcpy(&current_window->window_info,&window_info,sizeof(WINDOWINFO));
  current_window->title = _strdup(title);
  current_window->exe_path = _strdup(exe_path);
  current_window->delta_rc_client_height = delta_rc_client_height;
  current_window->delta_rc_client_width = delta_rc_client_width;
  current_window->delta_rc_window_height = delta_rc_window_height;
  current_window->delta_rc_window_width = delta_rc_window_width;
   
  if(current_window->title == NULL)
    {
      free(current_window);
      return(TRUE);
    }

  /* Stockage de la structure */
  my_Memory(MEMORY_ADD_WINDOWTITLE,current_window,NULL);

  return(TRUE);
}
/*************************************************************************************/